﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Termination {
  [Item("TerminationCriterion", "A base class for all termination criteria.")]
  [StorableClass]
  public abstract class TerminationCriterion : InstrumentedOperator, ITerminationCriterion {
    public ILookupParameter<BoolValue> TerminateParameter {
      get { return (ILookupParameter<BoolValue>)Parameters["Terminate"]; }
    }

    [StorableConstructor]
    protected TerminationCriterion(bool deserializing) : base(deserializing) { }
    protected TerminationCriterion(TerminationCriterion original, Cloner cloner) : base(original, cloner) { }

    protected TerminationCriterion()
      : base() {
      Parameters.Add(new LookupParameter<BoolValue>("Terminate", "The parameter which will be set to determine if execution should be terminated or should continue."));
    }

    public sealed override IOperation InstrumentedApply() {
      if (!TerminateParameter.ActualValue.Value) { // If terminate flag is already set, no need to check further termination criteria.
        bool terminate = CheckTermination();
        TerminateParameter.ActualValue = new BoolValue(terminate);
      }
      return base.InstrumentedApply();
    }

    protected abstract bool CheckTermination();
  }
}